# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import abc
import argparse
import re
import sys
from collections import defaultdict

from knack.log import get_logger
from knack.deprecation import Deprecated
from knack.util import StatusTag, color_map


logger = get_logger()

NEXT_BREAKING_CHANGE_RELEASE = '2.86.0'
NEXT_BREAKING_CHANGE_DATE = 'May 2026'
DEFAULT_BREAKING_CHANGE_TAG = '[Breaking Change]'


def _get_action_class(cli_ctx, action):
    action_class = argparse.Action

    # action is either a user-defined Action class or a string referring a library-defined Action
    if isinstance(action, type) and issubclass(action, argparse.Action):
        action_class = action
    elif isinstance(action, str):
        action_class = cli_ctx.invocation.parser._registries['action'][action]  # pylint: disable=protected-access
    return action_class


def _argument_breaking_change_action(cli_ctx, status_tag, action):

    action_class = _get_action_class(cli_ctx, action)

    class ArgumentBreakingChangeAction(action_class):

        def __call__(self, parser, namespace, values, option_string=None):
            if not hasattr(namespace, '_argument_deprecations'):
                setattr(namespace, '_argument_deprecations', [])
            if isinstance(status_tag, MergedStatusTag):
                for tag in status_tag.tags:
                    namespace._argument_deprecations.append(tag)
            else:
                if status_tag not in namespace._argument_deprecations:
                    namespace._argument_deprecations.append(status_tag)
            try:
                super().__call__(parser, namespace, values, option_string)
            except NotImplementedError:
                setattr(namespace, self.dest, values)

    return ArgumentBreakingChangeAction


def _find_arg(arg_name, arguments):
    if arg_name in arguments:
        return arg_name, arguments[arg_name]
    for key, argument in arguments.items():
        for option in argument.options_list or []:
            if arg_name == option or (isinstance(option, Deprecated) and arg_name == option.target):
                return key, argument
    trimmed_arg_name = arg_name.strip('-').replace('-', '_')
    if trimmed_arg_name in arguments:
        return trimmed_arg_name, arguments[trimmed_arg_name]
    return None, None


class UpcomingBreakingChangeTag(StatusTag):
    def __init__(self, cli_ctx, object_type='', target=None, target_version=None, tag_func=None, message_func=None,
                 always_display=False):
        def _default_get_message(bc):
            msg = f"A breaking change may occur to this {bc.object_type} "
            if isinstance(target_version, TargetVersion):
                msg += str(target_version) + '.'
            elif isinstance(target_version, str):
                msg += 'in ' + target_version + '.'
            else:
                msg += 'in future release.'
                return msg

        self.always_display = always_display
        self.target_version = target_version
        super().__init__(
            cli_ctx=cli_ctx,
            object_type=object_type,
            target=target,
            color=color_map['deprecation'],
            tag_func=tag_func or (lambda _: DEFAULT_BREAKING_CHANGE_TAG),
            message_func=message_func or _default_get_message
        )

    def expired(self):
        return False


class MergedStatusTag(StatusTag):
    # This class is used to merge multiple status tags into one.
    # It is particularly useful when multiple breaking changes and deprecation information need to be recorded
    # in a single deprecate_info field.

    def __init__(self, cli_ctx, *tags):
        assert len(tags) > 0
        tag = tags[0]
        self.tags = list(tags)

        def _get_merged_tag(self):
            return ''.join({tag._get_tag(tag) for tag in self.tags})  # pylint: disable=protected-access

        def _get_merged_msg(self):
            return '\n'.join({tag._get_message(tag) for tag in self.tags})  # pylint: disable=protected-access

        super().__init__(cli_ctx, tag.object_type, tag.target, tag_func=_get_merged_tag,
                         message_func=_get_merged_msg, color=tag._color)

    def merge(self, other):
        self.tags.append(other)

    def hidden(self):
        return any(tag.hidden() for tag in self.tags)

    def show_in_help(self):
        return any(tag.show_in_help() for tag in self.tags)

    def expired(self):
        return any(tag.expired() for tag in self.tags)

    @property
    def tag(self):
        return ''.join({str(tag.tag) for tag in self.tags})

    @property
    def message(self):
        return '\n'.join({str(tag.message) for tag in self.tags})


def _next_breaking_change_version():
    return NEXT_BREAKING_CHANGE_RELEASE


# pylint: disable=too-few-public-methods
class TargetVersion(abc.ABC):
    @abc.abstractmethod
    def __str__(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def version(self):
        raise NotImplementedError()


# pylint: disable=too-few-public-methods
class NextBreakingChangeWindow(TargetVersion):
    def __str__(self):
        next_breaking_change_version = _next_breaking_change_version()
        message = 'in next breaking change release'
        if next_breaking_change_version:
            message += f'({next_breaking_change_version})'
        if NEXT_BREAKING_CHANGE_DATE:
            message += f' scheduled for {NEXT_BREAKING_CHANGE_DATE}'
        return message

    def version(self):
        return _next_breaking_change_version()


# pylint: disable=too-few-public-methods
class ExactVersion(TargetVersion):
    def __init__(self, version):
        self._version = version

    def __str__(self):
        return f'in {self._version}'

    def version(self):
        return self._version


# pylint: disable=too-few-public-methods
class NonVersion(TargetVersion):
    def __init__(self, msg):
        self._msg = msg

    def __str__(self):
        return f'in {self._msg}'

    def version(self):
        return None


# pylint: disable=too-few-public-methods
class UnspecificVersion(TargetVersion):
    def __str__(self):
        return 'in a future release'

    def version(self):
        return None


class BreakingChange(abc.ABC):
    def __init__(self, cmd, arg=None, target=None, target_version=None):
        self.cmd = cmd
        if isinstance(arg, str):
            self.args = [arg]
        elif isinstance(arg, list):
            self.args = arg
        else:
            self.args = []
        self.target = target if target else '/'.join(self.args) if self.args else self.cmd
        if isinstance(target_version, TargetVersion):
            self._target_version = target_version
        elif isinstance(target_version, str) and re.match(r'\d+.\d+.\d+', target_version):
            self._target_version = ExactVersion(target_version)
        elif isinstance(target_version, str):
            self._target_version = NonVersion(target_version)
        else:
            self._target_version = UnspecificVersion()

    @property
    def message(self):
        return ''

    @property
    def target_version(self):
        return self._target_version

    @staticmethod
    def format_doc_link(doc_link):
        return f' To know more about the Breaking Change, please visit {doc_link}.' if doc_link else ''

    @property
    def command_name(self):
        if self.cmd.startswith('az '):
            return self.cmd[3:].strip()
        return self.cmd

    def is_command_group(self, cli_ctx):
        return self.command_name in cli_ctx.invocation.commands_loader.command_group_table

    def to_tag(self, cli_ctx, **kwargs):
        if self.args:
            object_type = 'argument'
        elif self.is_command_group(cli_ctx):
            object_type = 'command group'
        else:
            object_type = 'command'
        tag_kwargs = {
            'object_type': object_type,
            'target': self.target,
            'target_version': self.target_version,
            'message_func': lambda _: self.message,
        }
        tag_kwargs.update(kwargs)
        return UpcomingBreakingChangeTag(cli_ctx, **tag_kwargs)

    def register(self, cli_ctx):
        if self.args:
            command = cli_ctx.invocation.commands_loader.command_table.get(self.command_name)
            if not command:
                return
            for arg_name in self.args:
                if self._register_option_deprecate(cli_ctx, command.arguments, arg_name):
                    continue
                arg_name, arg = _find_arg(arg_name, command.arguments)
                if not arg:
                    continue
                arg.deprecate_info = self.appended_status_tag(cli_ctx, arg.deprecate_info, self.to_tag(cli_ctx))
                arg.action = _argument_breaking_change_action(cli_ctx, arg.deprecate_info, arg.options.get('action'))
        elif self.is_command_group(cli_ctx):
            command_group = cli_ctx.invocation.commands_loader.command_group_table[self.command_name]
            if not command_group:
                loader = self.find_suitable_command_loader(cli_ctx)
                if not loader:
                    return
                command_group = loader.command_group(self.command_name)
                cli_ctx.invocation.commands_loader.command_group_table[self.command_name] = command_group
            if command_group:
                command_group.group_kwargs['deprecate_info'] = \
                    self.appended_status_tag(cli_ctx, command_group.group_kwargs.get('deprecate_info'),
                                             self.to_tag(cli_ctx))
        else:
            command = cli_ctx.invocation.commands_loader.command_table.get(self.cmd)
            if not command:
                return
            command.deprecate_info = self.appended_status_tag(cli_ctx, command.deprecate_info, self.to_tag(cli_ctx))

    @staticmethod
    def appended_status_tag(cli_ctx, old_status_tag, new_status_tag):
        if isinstance(old_status_tag, (Deprecated, UpcomingBreakingChangeTag)):
            return MergedStatusTag(cli_ctx, old_status_tag, new_status_tag)
        if isinstance(old_status_tag, MergedStatusTag):
            old_status_tag.merge(new_status_tag)
            return old_status_tag
        return new_status_tag

    def find_suitable_command_loader(self, cli_ctx):
        for name, loader in cli_ctx.invocation.commands_loader.cmd_to_loader_map.items():
            if name.startswith(self.command_name) and loader:
                return loader[0]
        return None

    def _register_option_deprecate(self, cli_ctx, arguments, option_name):
        for _, argument in arguments.items():
            if argument.options_list and len(argument.options_list) > 1:
                for idx, option in enumerate(argument.options_list):
                    if isinstance(option, str) and option_name == option and isinstance(self, AzCLIDeprecate):
                        if isinstance(argument.options_list, tuple):
                            # Some of the command would declare options_list as tuple
                            argument.options_list = list(argument.options_list)
                        argument.options_list[idx] = self.to_tag(cli_ctx, object_type='option')
                        argument.options_list[idx].target = option
                        argument.action = _argument_breaking_change_action(cli_ctx, argument.options_list[idx],
                                                                           argument.options['action'])
                        return True
        return False


class AzCLIDeprecate(BreakingChange):
    def __init__(self, cmd, arg=None, target_version=NextBreakingChangeWindow(), message=None, **kwargs):
        super().__init__(cmd, arg, None, target_version)
        self._message = message
        self.kwargs = kwargs

    @staticmethod
    def _build_message(object_type, target, target_version, redirect):
        if object_type in ['argument', 'option']:
            msg = "{} '{}' has been deprecated and will be removed ".format(object_type, target).capitalize()
        elif object_type:
            msg = "This {} has been deprecated and will be removed ".format(object_type)
        else:
            msg = "'{}' has been deprecated and will be removed ".format(target)
        msg += str(target_version) + '.'
        if redirect:
            msg += " Use '{}' instead.".format(redirect)
        return msg

    @property
    def message(self):
        return self._message or self._build_message(self.kwargs.get('object_type'), self.target, self.target_version,
                                                    self.kwargs.get('redirect'))

    def to_tag(self, cli_ctx, **kwargs):
        if self.args:
            object_type = 'argument'
        elif self.is_command_group(cli_ctx):
            object_type = 'command group'
        else:
            object_type = 'command'
        tag_kwargs = {
            'object_type': object_type,
            'message_func': lambda depr: self._message or self._build_message(
                depr.object_type, depr.target, self.target_version, depr.redirect),
            'target': self.target
        }
        tag_kwargs.update(self.kwargs)
        tag_kwargs.update(kwargs)
        return Deprecated(cli_ctx, **tag_kwargs)


class AzCLIRemoveChange(BreakingChange):
    """
    Remove the command groups, commands or arguments in a future release.

    **It is recommended to utilize `deprecate_info` instead of this class to pre-announce Breaking Change of Removal.**
    :param target: name of the removed command group, command or argument
    :param target_version: version where the breaking change is expected to happen.
    :type target_version: TargetVersion
    :param redirect: alternative way to replace the old behavior
    :param doc_link: link of the related document
    """

    def __init__(self, cmd, arg=None, target_version=NextBreakingChangeWindow(), target=None, redirect=None,
                 doc_link=None):
        super().__init__(cmd, arg, target, target_version)
        self.alter = redirect
        self.doc_link = doc_link

    @property
    def message(self):
        alter = f" Please use '{self.alter}' instead." if self.alter else ''
        doc = self.format_doc_link(self.doc_link)
        return f"'{self.target}' will be removed {str(self._target_version)}.{alter}{doc}"


class AzCLIRenameChange(BreakingChange):
    """
    Rename the command groups, commands or arguments to a new name in a future release.

    **It is recommended to utilize `deprecate_info` instead of this class to pre-announce Breaking Change of Renaming.**
    It is recommended that the old name and the new name should be reserved in few releases.
    :param target: name of the renamed command group, command or argument
    :param new_name: new name
    :param target_version: version where the breaking change is expected to happen.
    :type target_version: TargetVersion
    :param doc_link: link of the related document
    """

    def __init__(self, cmd, new_name, arg=None, target=None, target_version=NextBreakingChangeWindow(), doc_link=None):
        super().__init__(cmd, arg, target, target_version)
        self.new_name = new_name
        self.doc_link = doc_link

    @property
    def message(self):
        doc = self.format_doc_link(self.doc_link)
        return f"'{self.target}' will be renamed to '{self.new_name}' {str(self._target_version)}.{doc}"


class AzCLIOutputChange(BreakingChange):
    """
    The output of the command will be changed in a future release.
    :param description: describe the changes in output
    :param target_version: version where the breaking change is expected to happen.
    :type target_version: TargetVersion
    :param guide: how to adapt to the change
    :param doc_link: link of the related document
    """

    def __init__(self, cmd, description: str, target_version=NextBreakingChangeWindow(), guide=None, doc_link=None):
        super().__init__(cmd, None, None, target_version)
        self.desc = description
        self.guide = guide
        self.doc_link = doc_link

    @property
    def message(self):
        desc = self.desc.rstrip()
        if desc and desc[-1] not in ',.;?!':
            desc = desc + '.'
        if self.guide:
            guide = self.guide.rstrip()
            if guide and guide[-1] not in ',.;?!':
                guide = ' ' + guide + '.'
        else:
            guide = ''
        doc = self.format_doc_link(self.doc_link)
        return f'The output will be changed {str(self.target_version)}. {desc}{guide}{doc}'


class AzCLILogicChange(BreakingChange):
    """
    There would be a breaking change in the logic of the command in future release.
    :param summary: a short summary about the breaking change
    :param target_version: version where the breaking change is expected to happen.
    :type target_version: TargetVersion
    :param detail: detailed information
    :param doc_link: link of the related document
    """

    def __init__(self, cmd, summary, target_version=NextBreakingChangeWindow(), detail=None, doc_link=None):
        super().__init__(cmd, None, None, target_version)
        self.summary = summary
        self.detail = detail
        self.doc_link = doc_link

    @property
    def message(self):
        detail = f' {self.detail}' if self.detail else ''
        return f'{self.summary} {str(self.target_version)}.{detail}{self.format_doc_link(self.doc_link)}'


class AzCLIDefaultChange(BreakingChange):
    """
    The default value of an argument would be changed in a future release.
    :param target: name of the related argument
    :param current_default: current default value of the argument
    :param new_default: new default value of the argument
    :param target_version: version where the breaking change is expected to happen.
    :type target_version: TargetVersion
    :param doc_link: link of the related document
    """

    def __init__(self, cmd, arg, current_default, new_default, target_version=NextBreakingChangeWindow(),
                 target=None, doc_link=None):
        super().__init__(cmd, arg, target, target_version)
        self.current_default = current_default
        self.new_default = new_default
        self.doc_link = doc_link

    @property
    def message(self):
        doc = self.format_doc_link(self.doc_link)
        return (f"The default value of '{self.target}' will be changed to '{self.new_default}' from "
                f"'{self.current_default}' {str(self._target_version)}.{doc}")

    def to_tag(self, cli_ctx, **kwargs):
        if 'always_display' not in kwargs:
            kwargs['always_display'] = True
        return super().to_tag(cli_ctx, **kwargs)


class AzCLIBeRequired(BreakingChange):
    """
    The argument would become required in a future release.
    :param target: name of the related argument
    :param target_version: version where the breaking change is expected to happen.
    :type target_version: TargetVersion
    :param doc_link: link of the related document
    """

    def __init__(self, cmd, arg, target_version=NextBreakingChangeWindow(), target=None, doc_link=None):
        super().__init__(cmd, arg, target, target_version)
        self.doc_link = doc_link

    @property
    def message(self):
        doc = self.format_doc_link(self.doc_link)
        return f"The argument '{self.target}' will become required {str(self._target_version)}.{doc}"

    def to_tag(self, cli_ctx, **kwargs):
        if 'always_display' not in kwargs:
            kwargs['always_display'] = True
        return super().to_tag(cli_ctx, **kwargs)


class AzCLIOtherChange(BreakingChange):
    """
    Other custom breaking changes.
    :param message: A description of the breaking change, including the version number where it is expected to occur.
    :param target_version: version where the breaking change is expected to happen.
    :type target_version: TargetVersion
    """

    def __init__(self, cmd, message, arg=None, target_version=NextBreakingChangeWindow()):
        super().__init__(cmd, arg, None, target_version)
        self._message = message

    @property
    def message(self):
        return self._message


upcoming_breaking_changes = defaultdict(lambda: [])


def import_core_breaking_changes():
    try:
        from importlib import import_module
        import_module('azure.cli.core._breaking_change')
    except ImportError:
        pass


def import_module_breaking_changes(mod):
    try:
        from importlib import import_module
        import_module('azure.cli.command_modules.' + mod + '._breaking_change')
    except ImportError:
        pass


def import_extension_breaking_changes(ext_mod):
    try:
        from importlib import import_module
        import_module(ext_mod + '._breaking_change')
    except ImportError:
        pass


def register_upcoming_breaking_change_info(cli_ctx):
    from knack import events

    def update_breaking_change_info(cli_ctx, **kwargs):  # pylint: disable=unused-argument
        for key, breaking_changes in upcoming_breaking_changes.items():
            # Conditional Breaking Changes are announced with key `CommandName.Tag`. They should not be registered.
            if '.' in key:
                continue
            for breaking_change in breaking_changes:
                breaking_change.register(cli_ctx)

    cli_ctx.register_event(events.EVENT_INVOKER_POST_CMD_TBL_CREATE, update_breaking_change_info)


def register_deprecate_info(command_name, arg=None, target_version=NextBreakingChangeWindow(), message=None, **kwargs):
    upcoming_breaking_changes[command_name].append(AzCLIDeprecate(command_name, arg, target_version, message, **kwargs))


def register_output_breaking_change(command_name, description, target_version=NextBreakingChangeWindow(), guide=None,
                                    doc_link=None):
    upcoming_breaking_changes[command_name].append(
        AzCLIOutputChange(command_name, description, target_version, guide, doc_link))


def register_logic_breaking_change(command_name, summary, target_version=NextBreakingChangeWindow(), detail=None,
                                   doc_link=None):
    upcoming_breaking_changes[command_name].append(
        AzCLILogicChange(command_name, summary, target_version, detail, doc_link))


def register_default_value_breaking_change(command_name, arg, current_default, new_default,
                                           target_version=NextBreakingChangeWindow(), target=None, doc_link=None):
    upcoming_breaking_changes[command_name].append(
        AzCLIDefaultChange(command_name, arg, current_default, new_default, target_version, target, doc_link))


def register_required_flag_breaking_change(command_name, arg, target_version=NextBreakingChangeWindow(), target=None,
                                           doc_link=None):
    upcoming_breaking_changes[command_name].append(AzCLIBeRequired(command_name, arg, target_version, target, doc_link))


def register_other_breaking_change(command_name, message, arg=None, target_version=NextBreakingChangeWindow()):
    upcoming_breaking_changes[command_name].append(AzCLIOtherChange(command_name, message, arg, target_version))


def register_command_group_deprecate(command_group, redirect=None, hide=None,
                                     target_version=NextBreakingChangeWindow(), message=None, **kwargs):
    register_deprecate_info(command_group, redirect=redirect, hide=hide, target_version=target_version,
                            message=message, **kwargs)


def register_command_deprecate(command, redirect=None, hide=None,
                               target_version=NextBreakingChangeWindow(), message=None, **kwargs):
    register_deprecate_info(command, redirect=redirect, hide=hide, target_version=target_version,
                            message=message, **kwargs)


def register_argument_deprecate(command, argument, redirect=None, hide=None,
                                target_version=NextBreakingChangeWindow(), message=None, **kwargs):
    register_deprecate_info(command, argument, redirect=redirect, hide=hide, target_version=target_version,
                            message=message, **kwargs)


def register_conditional_breaking_change(tag, breaking_change, *, command_name=None):
    if command_name and command_name.startswith('az '):
        command_name = command_name[3:].strip()
    if isinstance(breaking_change, BreakingChange):
        command_name = command_name or breaking_change.command_name
        upcoming_breaking_changes[command_name + '.' + tag].append(breaking_change)
    elif isinstance(breaking_change, str):
        command_name = command_name or ''
        upcoming_breaking_changes[command_name + '.' + tag].append(AzCLIOtherChange(
            cmd=command_name,
            message=breaking_change,
        ))


def print_conditional_breaking_change(cli_ctx, tag, *, custom_logger=None, command_name=None):
    """
    Print a breaking change warning message manually.
    :param cli_ctx: By default, retrieve the command name from cli_ctx.
    :param tag: Use the tag to distinguish different warning messages to be printed in the same command.
    Please note, all breaking change items with the same tag from the parent command group will also be printed.
    :param custom_logger: Use a custom logger to replace the logger in azure.cli.core.breaking_change.
    :param command_name: Specify the command name if not pass in the cli_ctx
    """
    command = cli_ctx.invocation.commands_loader.command_name if cli_ctx else command_name
    command = command or ''
    tag_suffix = '.' + tag if tag is not None else ''

    command_comps = command.split()
    while command_comps:
        for breaking_change in upcoming_breaking_changes.get(' '.join(command_comps) + tag_suffix, []):
            if custom_logger:
                custom_logger.warning(breaking_change.message)
            else:
                print(breaking_change.message, file=sys.stderr)
        del command_comps[-1]
    for breaking_change in upcoming_breaking_changes.get(tag_suffix, []):
        if custom_logger:
            custom_logger.warning(breaking_change.message)
        else:
            print(breaking_change.message, file=sys.stderr)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AddressDetails
    from ._models_py3 import Agreement
    from ._models_py3 import AgreementListResult
    from ._models_py3 import Amount
    from ._models_py3 import AvailableBalance
    from ._models_py3 import AzurePlan
    from ._models_py3 import BillingAccount
    from ._models_py3 import BillingAccountListResult
    from ._models_py3 import BillingAccountUpdateRequest
    from ._models_py3 import BillingPeriod
    from ._models_py3 import BillingPeriodsListResult
    from ._models_py3 import BillingPermissionsListResult
    from ._models_py3 import BillingPermissionsProperties
    from ._models_py3 import BillingProfile
    from ._models_py3 import BillingProfileCreationRequest
    from ._models_py3 import BillingProfileListResult
    from ._models_py3 import BillingProfilesOnExpand
    from ._models_py3 import BillingProperty
    from ._models_py3 import BillingRoleAssignment
    from ._models_py3 import BillingRoleAssignmentListResult
    from ._models_py3 import BillingRoleDefinition
    from ._models_py3 import BillingRoleDefinitionListResult
    from ._models_py3 import BillingSubscription
    from ._models_py3 import BillingSubscriptionsListResult
    from ._models_py3 import Customer
    from ._models_py3 import CustomerListResult
    from ._models_py3 import CustomerPolicy
    from ._models_py3 import Department
    from ._models_py3 import Document
    from ._models_py3 import DownloadUrl
    from ._models_py3 import Enrollment
    from ._models_py3 import EnrollmentAccount
    from ._models_py3 import EnrollmentAccountContext
    from ._models_py3 import EnrollmentAccountListResult
    from ._models_py3 import EnrollmentAccountSummary
    from ._models_py3 import EnrollmentPolicies
    from ._models_py3 import ErrorDetails
    from ._models_py3 import ErrorResponse
    from ._models_py3 import ErrorSubDetailsItem
    from ._models_py3 import IndirectRelationshipInfo
    from ._models_py3 import Instruction
    from ._models_py3 import InstructionListResult
    from ._models_py3 import Invoice
    from ._models_py3 import InvoiceListResult
    from ._models_py3 import InvoiceSection
    from ._models_py3 import InvoiceSectionCreationRequest
    from ._models_py3 import InvoiceSectionListResult
    from ._models_py3 import InvoiceSectionListWithCreateSubPermissionResult
    from ._models_py3 import InvoiceSectionWithCreateSubPermission
    from ._models_py3 import InvoiceSectionsOnExpand
    from ._models_py3 import Operation
    from ._models_py3 import OperationDisplay
    from ._models_py3 import OperationListResult
    from ._models_py3 import Participants
    from ._models_py3 import PaymentProperties
    from ._models_py3 import Policy
    from ._models_py3 import Product
    from ._models_py3 import ProductsListResult
    from ._models_py3 import RebillDetails
    from ._models_py3 import Reseller
    from ._models_py3 import Reservation
    from ._models_py3 import ReservationPropertyUtilization
    from ._models_py3 import ReservationSkuProperty
    from ._models_py3 import ReservationSummary
    from ._models_py3 import ReservationUtilizationAggregates
    from ._models_py3 import ReservationsListResult
    from ._models_py3 import Resource
    from ._models_py3 import Transaction
    from ._models_py3 import TransactionListResult
    from ._models_py3 import TransferBillingSubscriptionRequestProperties
    from ._models_py3 import TransferProductRequestProperties
    from ._models_py3 import ValidateAddressResponse
    from ._models_py3 import ValidateProductTransferEligibilityError
    from ._models_py3 import ValidateProductTransferEligibilityResult
    from ._models_py3 import ValidateSubscriptionTransferEligibilityError
    from ._models_py3 import ValidateSubscriptionTransferEligibilityResult
except (SyntaxError, ImportError):
    from ._models import AddressDetails  # type: ignore
    from ._models import Agreement  # type: ignore
    from ._models import AgreementListResult  # type: ignore
    from ._models import Amount  # type: ignore
    from ._models import AvailableBalance  # type: ignore
    from ._models import AzurePlan  # type: ignore
    from ._models import BillingAccount  # type: ignore
    from ._models import BillingAccountListResult  # type: ignore
    from ._models import BillingAccountUpdateRequest  # type: ignore
    from ._models import BillingPeriod  # type: ignore
    from ._models import BillingPeriodsListResult  # type: ignore
    from ._models import BillingPermissionsListResult  # type: ignore
    from ._models import BillingPermissionsProperties  # type: ignore
    from ._models import BillingProfile  # type: ignore
    from ._models import BillingProfileCreationRequest  # type: ignore
    from ._models import BillingProfileListResult  # type: ignore
    from ._models import BillingProfilesOnExpand  # type: ignore
    from ._models import BillingProperty  # type: ignore
    from ._models import BillingRoleAssignment  # type: ignore
    from ._models import BillingRoleAssignmentListResult  # type: ignore
    from ._models import BillingRoleDefinition  # type: ignore
    from ._models import BillingRoleDefinitionListResult  # type: ignore
    from ._models import BillingSubscription  # type: ignore
    from ._models import BillingSubscriptionsListResult  # type: ignore
    from ._models import Customer  # type: ignore
    from ._models import CustomerListResult  # type: ignore
    from ._models import CustomerPolicy  # type: ignore
    from ._models import Department  # type: ignore
    from ._models import Document  # type: ignore
    from ._models import DownloadUrl  # type: ignore
    from ._models import Enrollment  # type: ignore
    from ._models import EnrollmentAccount  # type: ignore
    from ._models import EnrollmentAccountContext  # type: ignore
    from ._models import EnrollmentAccountListResult  # type: ignore
    from ._models import EnrollmentAccountSummary  # type: ignore
    from ._models import EnrollmentPolicies  # type: ignore
    from ._models import ErrorDetails  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import ErrorSubDetailsItem  # type: ignore
    from ._models import IndirectRelationshipInfo  # type: ignore
    from ._models import Instruction  # type: ignore
    from ._models import InstructionListResult  # type: ignore
    from ._models import Invoice  # type: ignore
    from ._models import InvoiceListResult  # type: ignore
    from ._models import InvoiceSection  # type: ignore
    from ._models import InvoiceSectionCreationRequest  # type: ignore
    from ._models import InvoiceSectionListResult  # type: ignore
    from ._models import InvoiceSectionListWithCreateSubPermissionResult  # type: ignore
    from ._models import InvoiceSectionWithCreateSubPermission  # type: ignore
    from ._models import InvoiceSectionsOnExpand  # type: ignore
    from ._models import Operation  # type: ignore
    from ._models import OperationDisplay  # type: ignore
    from ._models import OperationListResult  # type: ignore
    from ._models import Participants  # type: ignore
    from ._models import PaymentProperties  # type: ignore
    from ._models import Policy  # type: ignore
    from ._models import Product  # type: ignore
    from ._models import ProductsListResult  # type: ignore
    from ._models import RebillDetails  # type: ignore
    from ._models import Reseller  # type: ignore
    from ._models import Reservation  # type: ignore
    from ._models import ReservationPropertyUtilization  # type: ignore
    from ._models import ReservationSkuProperty  # type: ignore
    from ._models import ReservationSummary  # type: ignore
    from ._models import ReservationUtilizationAggregates  # type: ignore
    from ._models import ReservationsListResult  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import Transaction  # type: ignore
    from ._models import TransactionListResult  # type: ignore
    from ._models import TransferBillingSubscriptionRequestProperties  # type: ignore
    from ._models import TransferProductRequestProperties  # type: ignore
    from ._models import ValidateAddressResponse  # type: ignore
    from ._models import ValidateProductTransferEligibilityError  # type: ignore
    from ._models import ValidateProductTransferEligibilityResult  # type: ignore
    from ._models import ValidateSubscriptionTransferEligibilityError  # type: ignore
    from ._models import ValidateSubscriptionTransferEligibilityResult  # type: ignore

from ._billing_management_client_enums import (
    AcceptanceMode,
    AccountStatus,
    AccountType,
    AddressValidationStatus,
    AgreementType,
    AutoRenew,
    BillingFrequency,
    BillingProfileSpendingLimit,
    BillingProfileStatus,
    BillingProfileStatusReasonCode,
    BillingRelationshipType,
    BillingSubscriptionStatusType,
    Category,
    DocumentSource,
    DocumentType,
    InvoiceDocumentType,
    InvoiceSectionState,
    InvoiceStatus,
    InvoiceType,
    MarketplacePurchasesPolicy,
    PaymentMethodFamily,
    ProductStatusType,
    ProductTransferValidationErrorCode,
    ReservationPurchasesPolicy,
    ReservationType,
    SpendingLimit,
    SpendingLimitForBillingProfile,
    StatusReasonCode,
    StatusReasonCodeForBillingProfile,
    SubscriptionTransferValidationErrorCode,
    TargetCloud,
    TransactionTypeKind,
    ViewCharges,
    ViewChargesPolicy,
)

__all__ = [
    'AddressDetails',
    'Agreement',
    'AgreementListResult',
    'Amount',
    'AvailableBalance',
    'AzurePlan',
    'BillingAccount',
    'BillingAccountListResult',
    'BillingAccountUpdateRequest',
    'BillingPeriod',
    'BillingPeriodsListResult',
    'BillingPermissionsListResult',
    'BillingPermissionsProperties',
    'BillingProfile',
    'BillingProfileCreationRequest',
    'BillingProfileListResult',
    'BillingProfilesOnExpand',
    'BillingProperty',
    'BillingRoleAssignment',
    'BillingRoleAssignmentListResult',
    'BillingRoleDefinition',
    'BillingRoleDefinitionListResult',
    'BillingSubscription',
    'BillingSubscriptionsListResult',
    'Customer',
    'CustomerListResult',
    'CustomerPolicy',
    'Department',
    'Document',
    'DownloadUrl',
    'Enrollment',
    'EnrollmentAccount',
    'EnrollmentAccountContext',
    'EnrollmentAccountListResult',
    'EnrollmentAccountSummary',
    'EnrollmentPolicies',
    'ErrorDetails',
    'ErrorResponse',
    'ErrorSubDetailsItem',
    'IndirectRelationshipInfo',
    'Instruction',
    'InstructionListResult',
    'Invoice',
    'InvoiceListResult',
    'InvoiceSection',
    'InvoiceSectionCreationRequest',
    'InvoiceSectionListResult',
    'InvoiceSectionListWithCreateSubPermissionResult',
    'InvoiceSectionWithCreateSubPermission',
    'InvoiceSectionsOnExpand',
    'Operation',
    'OperationDisplay',
    'OperationListResult',
    'Participants',
    'PaymentProperties',
    'Policy',
    'Product',
    'ProductsListResult',
    'RebillDetails',
    'Reseller',
    'Reservation',
    'ReservationPropertyUtilization',
    'ReservationSkuProperty',
    'ReservationSummary',
    'ReservationUtilizationAggregates',
    'ReservationsListResult',
    'Resource',
    'Transaction',
    'TransactionListResult',
    'TransferBillingSubscriptionRequestProperties',
    'TransferProductRequestProperties',
    'ValidateAddressResponse',
    'ValidateProductTransferEligibilityError',
    'ValidateProductTransferEligibilityResult',
    'ValidateSubscriptionTransferEligibilityError',
    'ValidateSubscriptionTransferEligibilityResult',
    'AcceptanceMode',
    'AccountStatus',
    'AccountType',
    'AddressValidationStatus',
    'AgreementType',
    'AutoRenew',
    'BillingFrequency',
    'BillingProfileSpendingLimit',
    'BillingProfileStatus',
    'BillingProfileStatusReasonCode',
    'BillingRelationshipType',
    'BillingSubscriptionStatusType',
    'Category',
    'DocumentSource',
    'DocumentType',
    'InvoiceDocumentType',
    'InvoiceSectionState',
    'InvoiceStatus',
    'InvoiceType',
    'MarketplacePurchasesPolicy',
    'PaymentMethodFamily',
    'ProductStatusType',
    'ProductTransferValidationErrorCode',
    'ReservationPurchasesPolicy',
    'ReservationType',
    'SpendingLimit',
    'SpendingLimitForBillingProfile',
    'StatusReasonCode',
    'StatusReasonCodeForBillingProfile',
    'SubscriptionTransferValidationErrorCode',
    'TargetCloud',
    'TransactionTypeKind',
    'ViewCharges',
    'ViewChargesPolicy',
]



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools
from typing import Any, Callable, Dict, Generic, Iterable, Optional, TypeVar, Union
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.polling import LROPoller, NoPolling, PollingMethod
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.arm_polling import ARMPolling
from msrest import Serializer

from .. import models as _models
from .._vendor import _convert_request, _format_url_section
T = TypeVar('T')
JSONType = Any
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False

def build_list_request(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_get_request(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    live_event_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
        "liveEventName": _SERIALIZER.url("live_event_name", live_event_name, 'str', max_length=32, min_length=1, pattern=r'^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_create_request_initial(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    live_event_name: str,
    *,
    json: JSONType = None,
    content: Any = None,
    auto_start: Optional[bool] = None,
    **kwargs: Any
) -> HttpRequest:
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
        "liveEventName": _SERIALIZER.url("live_event_name", live_event_name, 'str', max_length=32, min_length=1, pattern=r'^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')
    if auto_start is not None:
        query_parameters['autoStart'] = _SERIALIZER.query("auto_start", auto_start, 'bool')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="PUT",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        json=json,
        content=content,
        **kwargs
    )


def build_update_request_initial(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    live_event_name: str,
    *,
    json: JSONType = None,
    content: Any = None,
    **kwargs: Any
) -> HttpRequest:
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
        "liveEventName": _SERIALIZER.url("live_event_name", live_event_name, 'str', max_length=32, min_length=1, pattern=r'^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="PATCH",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        json=json,
        content=content,
        **kwargs
    )


def build_delete_request_initial(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    live_event_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
        "liveEventName": _SERIALIZER.url("live_event_name", live_event_name, 'str', max_length=32, min_length=1, pattern=r'^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="DELETE",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_allocate_request_initial(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    live_event_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/allocate')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
        "liveEventName": _SERIALIZER.url("live_event_name", live_event_name, 'str', max_length=32, min_length=1, pattern=r'^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_start_request_initial(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    live_event_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/start')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
        "liveEventName": _SERIALIZER.url("live_event_name", live_event_name, 'str', max_length=32, min_length=1, pattern=r'^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_stop_request_initial(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    live_event_name: str,
    *,
    json: JSONType = None,
    content: Any = None,
    **kwargs: Any
) -> HttpRequest:
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/stop')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
        "liveEventName": _SERIALIZER.url("live_event_name", live_event_name, 'str', max_length=32, min_length=1, pattern=r'^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        json=json,
        content=content,
        **kwargs
    )


def build_reset_request_initial(
    subscription_id: str,
    resource_group_name: str,
    account_name: str,
    live_event_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = "2021-11-01"
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/reset')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "accountName": _SERIALIZER.url("account_name", account_name, 'str'),
        "liveEventName": _SERIALIZER.url("live_event_name", live_event_name, 'str', max_length=32, min_length=1, pattern=r'^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )

class LiveEventsOperations(object):
    """LiveEventsOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.media.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace
    def list(
        self,
        resource_group_name: str,
        account_name: str,
        **kwargs: Any
    ) -> Iterable["_models.LiveEventListResult"]:
        """List live events.

        Lists all the live events in the account.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either LiveEventListResult or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.media.models.LiveEventListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.LiveEventListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_list_request(
                    subscription_id=self._config.subscription_id,
                    resource_group_name=resource_group_name,
                    account_name=account_name,
                    template_url=self.list.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_list_request(
                    subscription_id=self._config.subscription_id,
                    resource_group_name=resource_group_name,
                    account_name=account_name,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("LiveEventListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.odata_next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response


        return ItemPaged(
            get_next, extract_data
        )
    list.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents'}  # type: ignore

    @distributed_trace
    def get(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> "_models.LiveEvent":
        """Get Live Event.

        Gets properties of a live event.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :param live_event_name: The name of the live event, maximum length is 32.
        :type live_event_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: LiveEvent, or the result of cls(response)
        :rtype: ~azure.mgmt.media.models.LiveEvent
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.LiveEvent"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_get_request(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            account_name=account_name,
            live_event_name=live_event_name,
            template_url=self.get.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('LiveEvent', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}'}  # type: ignore


    def _create_initial(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        parameters: "_models.LiveEvent",
        auto_start: Optional[bool] = None,
        **kwargs: Any
    ) -> "_models.LiveEvent":
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.LiveEvent"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _json = self._serialize.body(parameters, 'LiveEvent')

        request = build_create_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            account_name=account_name,
            live_event_name=live_event_name,
            content_type=content_type,
            json=_json,
            auto_start=auto_start,
            template_url=self._create_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize('LiveEvent', pipeline_response)

        if response.status_code == 201:
            deserialized = self._deserialize('LiveEvent', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    _create_initial.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}'}  # type: ignore


    @distributed_trace
    def begin_create(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        parameters: "_models.LiveEvent",
        auto_start: Optional[bool] = None,
        **kwargs: Any
    ) -> LROPoller["_models.LiveEvent"]:
        """Create Live Event.

        Creates a new live event.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :param live_event_name: The name of the live event, maximum length is 32.
        :type live_event_name: str
        :param parameters: Live event properties needed for creation.
        :type parameters: ~azure.mgmt.media.models.LiveEvent
        :param auto_start: The flag indicates if the resource should be automatically started on
         creation.
        :type auto_start: bool
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either LiveEvent or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.media.models.LiveEvent]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]
        polling = kwargs.pop('polling', True)  # type: Union[bool, azure.core.polling.PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.LiveEvent"]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._create_initial(
                resource_group_name=resource_group_name,
                account_name=account_name,
                live_event_name=live_event_name,
                parameters=parameters,
                auto_start=auto_start,
                content_type=content_type,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            response = pipeline_response.http_response
            deserialized = self._deserialize('LiveEvent', pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        else:
            return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_create.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}'}  # type: ignore

    def _update_initial(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        parameters: "_models.LiveEvent",
        **kwargs: Any
    ) -> "_models.LiveEvent":
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.LiveEvent"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _json = self._serialize.body(parameters, 'LiveEvent')

        request = build_update_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            account_name=account_name,
            live_event_name=live_event_name,
            content_type=content_type,
            json=_json,
            template_url=self._update_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize('LiveEvent', pipeline_response)

        if response.status_code == 202:
            deserialized = self._deserialize('LiveEvent', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    _update_initial.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}'}  # type: ignore


    @distributed_trace
    def begin_update(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        parameters: "_models.LiveEvent",
        **kwargs: Any
    ) -> LROPoller["_models.LiveEvent"]:
        """Updates settings on an existing live event.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :param live_event_name: The name of the live event, maximum length is 32.
        :type live_event_name: str
        :param parameters: Live event properties needed for patch.
        :type parameters: ~azure.mgmt.media.models.LiveEvent
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either LiveEvent or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.media.models.LiveEvent]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]
        polling = kwargs.pop('polling', True)  # type: Union[bool, azure.core.polling.PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.LiveEvent"]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._update_initial(
                resource_group_name=resource_group_name,
                account_name=account_name,
                live_event_name=live_event_name,
                parameters=parameters,
                content_type=content_type,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            response = pipeline_response.http_response
            deserialized = self._deserialize('LiveEvent', pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        else:
            return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_update.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}'}  # type: ignore

    def _delete_initial(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> None:
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_delete_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            account_name=account_name,
            live_event_name=live_event_name,
            template_url=self._delete_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    _delete_initial.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}'}  # type: ignore


    @distributed_trace
    def begin_delete(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> LROPoller[None]:
        """Delete Live Event.

        Deletes a live event.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :param live_event_name: The name of the live event, maximum length is 32.
        :type live_event_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        polling = kwargs.pop('polling', True)  # type: Union[bool, azure.core.polling.PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._delete_initial(
                resource_group_name=resource_group_name,
                account_name=account_name,
                live_event_name=live_event_name,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            if cls:
                return cls(pipeline_response, None, {})


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        else:
            return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_delete.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}'}  # type: ignore

    def _allocate_initial(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> None:
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_allocate_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            account_name=account_name,
            live_event_name=live_event_name,
            template_url=self._allocate_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    _allocate_initial.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/allocate'}  # type: ignore


    @distributed_trace
    def begin_allocate(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> LROPoller[None]:
        """Allocate resources for a live event.

        A live event is in StandBy state after allocation completes, and is ready to start.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :param live_event_name: The name of the live event, maximum length is 32.
        :type live_event_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        polling = kwargs.pop('polling', True)  # type: Union[bool, azure.core.polling.PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._allocate_initial(
                resource_group_name=resource_group_name,
                account_name=account_name,
                live_event_name=live_event_name,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            if cls:
                return cls(pipeline_response, None, {})


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        else:
            return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_allocate.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/allocate'}  # type: ignore

    def _start_initial(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> None:
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_start_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            account_name=account_name,
            live_event_name=live_event_name,
            template_url=self._start_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    _start_initial.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/start'}  # type: ignore


    @distributed_trace
    def begin_start(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> LROPoller[None]:
        """Start Live Event.

        A live event in Stopped or StandBy state will be in Running state after the start operation
        completes.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :param live_event_name: The name of the live event, maximum length is 32.
        :type live_event_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        polling = kwargs.pop('polling', True)  # type: Union[bool, azure.core.polling.PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._start_initial(
                resource_group_name=resource_group_name,
                account_name=account_name,
                live_event_name=live_event_name,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            if cls:
                return cls(pipeline_response, None, {})


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        else:
            return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_start.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/start'}  # type: ignore

    def _stop_initial(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        parameters: "_models.LiveEventActionInput",
        **kwargs: Any
    ) -> None:
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _json = self._serialize.body(parameters, 'LiveEventActionInput')

        request = build_stop_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            account_name=account_name,
            live_event_name=live_event_name,
            content_type=content_type,
            json=_json,
            template_url=self._stop_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    _stop_initial.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/stop'}  # type: ignore


    @distributed_trace
    def begin_stop(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        parameters: "_models.LiveEventActionInput",
        **kwargs: Any
    ) -> LROPoller[None]:
        """Stop Live Event.

        Stops a running live event.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :param live_event_name: The name of the live event, maximum length is 32.
        :type live_event_name: str
        :param parameters: LiveEvent stop parameters.
        :type parameters: ~azure.mgmt.media.models.LiveEventActionInput
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]
        polling = kwargs.pop('polling', True)  # type: Union[bool, azure.core.polling.PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._stop_initial(
                resource_group_name=resource_group_name,
                account_name=account_name,
                live_event_name=live_event_name,
                parameters=parameters,
                content_type=content_type,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            if cls:
                return cls(pipeline_response, None, {})


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        else:
            return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_stop.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/stop'}  # type: ignore

    def _reset_initial(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> None:
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_reset_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            account_name=account_name,
            live_event_name=live_event_name,
            template_url=self._reset_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    _reset_initial.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/reset'}  # type: ignore


    @distributed_trace
    def begin_reset(
        self,
        resource_group_name: str,
        account_name: str,
        live_event_name: str,
        **kwargs: Any
    ) -> LROPoller[None]:
        """Reset Live Event.

        Resets an existing live event. All live outputs for the live event are deleted and the live
        event is stopped and will be started again. All assets used by the live outputs and streaming
        locators created on these assets are unaffected.

        :param resource_group_name: The name of the resource group within the Azure subscription.
        :type resource_group_name: str
        :param account_name: The Media Services account name.
        :type account_name: str
        :param live_event_name: The name of the live event, maximum length is 32.
        :type live_event_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        polling = kwargs.pop('polling', True)  # type: Union[bool, azure.core.polling.PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._reset_initial(
                resource_group_name=resource_group_name,
                account_name=account_name,
                live_event_name=live_event_name,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            if cls:
                return cls(pipeline_response, None, {})


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        else:
            return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_reset.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Media/mediaservices/{accountName}/liveEvents/{liveEventName}/reset'}  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import ErrorResponse
    from ._models_py3 import LiveTokenResponse
    from ._models_py3 import OperationInfo
    from ._models_py3 import OperationLive
    from ._models_py3 import OperationsListResult
except (SyntaxError, ImportError):
    from ._models import ErrorResponse  # type: ignore
    from ._models import LiveTokenResponse  # type: ignore
    from ._models import OperationInfo  # type: ignore
    from ._models import OperationLive  # type: ignore
    from ._models import OperationsListResult  # type: ignore

__all__ = [
    'ErrorResponse',
    'LiveTokenResponse',
    'OperationInfo',
    'OperationLive',
    'OperationsListResult',
]



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class CloudServiceSlotType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Slot type for the cloud service.
    Possible values are :code:`<br />`\\ :code:`<br />`\\ **Production**\\ :code:`<br />`\\
    :code:`<br />`\\ **Staging**\\ :code:`<br />`\\ :code:`<br />`
    If not specified, the default value is Production.
    """

    PRODUCTION = "Production"
    STAGING = "Staging"


class CloudServiceUpgradeMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Update mode for the cloud service. Role instances are allocated to update domains when the
    service is deployed. Updates can be initiated manually in each update domain or initiated
    automatically in all update domains.
    Possible Values are :code:`<br />`\\ :code:`<br />`\\ **Auto**\\ :code:`<br />`\\ :code:`<br
    />`\\ **Manual** :code:`<br />`\\ :code:`<br />`\\ **Simultaneous**\\ :code:`<br />`\\
    :code:`<br />`
    If not specified, the default value is Auto. If set to Manual, PUT UpdateDomain must be called
    to apply the update. If set to Auto, the update is automatically applied to each update domain
    in sequence.
    """

    AUTO = "Auto"
    MANUAL = "Manual"
    SIMULTANEOUS = "Simultaneous"


class ExtendedLocationTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of extendedLocation."""

    EDGE_ZONE = "EdgeZone"


class InstanceViewTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InstanceViewTypes."""

    INSTANCE_VIEW = "instanceView"
    USER_DATA = "userData"


class StatusLevelTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The level code."""

    INFO = "Info"
    WARNING = "Warning"
    ERROR = "Error"



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class __CMDGroup(AAZCommandGroup):
    """Manage the managed data disks attached to a VM.

    Azure Virtual Machines use disks as a place to store an operating system, applications, and
            data. All Azure virtual machines have at least two disks: An operating system disk, and a
            temporary disk. The operating system disk is created from an image, and both the operating
            system disk and the image are actually virtual hard disks (VHDs) stored in an Azure storage
            account. Virtual machines also can have one or more data disks, that are also stored as
            VHDs.
            Azure Managed and Unmanaged Data Disks have a maximum size of 4095 GB (with the exception of
            larger disks in preview). Azure Unmanaged Disks also have a maximum capacity of 4095 GB.
            For more information, see:
            - Azure Disks - https://learn.microsoft.com/azure/virtual-machines/managed-disks-overview.
            - Larger Managed Disks in Public Preview - https://azure.microsoft.com/blog/introducing-the-
            public-preview-of-larger-managed-disks-sizes/
            - Ultra SSD Managed Disks in Public Preview - https://learn.microsoft.com/azure/virtual-
            machines/disks-types.
    """
    pass


__all__ = ["__CMDGroup"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.core.breaking_change import (
    register_argument_deprecate,
    register_default_value_breaking_change,
    register_other_breaking_change,
)
register_argument_deprecate('netappfiles volume update', '--remote-volume-resource-id')
register_argument_deprecate('netappfiles volume create', '--default-group-quota',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume update', '--default-group-quota',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume create', '--default-user-quota',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume update', '--default-user-quota',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume create', '--is-default-quota-enabled',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume update', '--is-default-quota-enabled',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume create', '--default-group-quota-in-ki-bs',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume update', '--default-group-quota-in-ki-bs',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume create', '--default-user-quota-in-ki-bs',
                            redirect='command group az netappfiles volume quota-rule')
register_argument_deprecate('netappfiles volume update', '--default-user-quota-in-ki-bs',
                            redirect='command group az netappfiles volume quota-rule')
register_default_value_breaking_change('netappfiles volume create', '--network-features', 'Basic', 'Standard')
register_other_breaking_change('netappfiles volume create',
                               'The basic option will not be accepted, use Standard instead',
                               arg='--network-features')



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import ARMErrorResponseBody
    from ._models_py3 import ArmErrorResponse
    from ._models_py3 import ConfigData
    from ._models_py3 import ConfigurationListResult
    from ._models_py3 import DigestConfig
    from ._models_py3 import MetadataEntity
    from ._models_py3 import MetadataEntityListResult
    from ._models_py3 import MetadataSupportedValueDetail
    from ._models_py3 import OperationDisplayInfo
    from ._models_py3 import OperationEntity
    from ._models_py3 import OperationEntityListResult
    from ._models_py3 import Resource
    from ._models_py3 import ResourceMetadata
    from ._models_py3 import ResourceRecommendationBase
    from ._models_py3 import ResourceRecommendationBaseListResult
    from ._models_py3 import ShortDescription
    from ._models_py3 import SuppressionContract
    from ._models_py3 import SuppressionContractListResult
except (SyntaxError, ImportError):
    from ._models import ARMErrorResponseBody  # type: ignore
    from ._models import ArmErrorResponse  # type: ignore
    from ._models import ConfigData  # type: ignore
    from ._models import ConfigurationListResult  # type: ignore
    from ._models import DigestConfig  # type: ignore
    from ._models import MetadataEntity  # type: ignore
    from ._models import MetadataEntityListResult  # type: ignore
    from ._models import MetadataSupportedValueDetail  # type: ignore
    from ._models import OperationDisplayInfo  # type: ignore
    from ._models import OperationEntity  # type: ignore
    from ._models import OperationEntityListResult  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ResourceMetadata  # type: ignore
    from ._models import ResourceRecommendationBase  # type: ignore
    from ._models import ResourceRecommendationBaseListResult  # type: ignore
    from ._models import ShortDescription  # type: ignore
    from ._models import SuppressionContract  # type: ignore
    from ._models import SuppressionContractListResult  # type: ignore

from ._advisor_management_client_enums import (
    Category,
    ConfigurationName,
    CpuThreshold,
    DigestConfigState,
    Impact,
    Risk,
    Scenario,
)

__all__ = [
    'ARMErrorResponseBody',
    'ArmErrorResponse',
    'ConfigData',
    'ConfigurationListResult',
    'DigestConfig',
    'MetadataEntity',
    'MetadataEntityListResult',
    'MetadataSupportedValueDetail',
    'OperationDisplayInfo',
    'OperationEntity',
    'OperationEntityListResult',
    'Resource',
    'ResourceMetadata',
    'ResourceRecommendationBase',
    'ResourceRecommendationBaseListResult',
    'ShortDescription',
    'SuppressionContract',
    'SuppressionContractListResult',
    'Category',
    'ConfigurationName',
    'CpuThreshold',
    'DigestConfigState',
    'Impact',
    'Risk',
    'Scenario',
]



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import Any, AsyncIterable, Callable, Dict, Generic, Optional, TypeVar, Union
import warnings

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse, HttpRequest
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models

T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class CreatorsOperations:
    """CreatorsOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.maps.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    def list_by_account(
        self,
        resource_group_name: str,
        account_name: str,
        **kwargs
    ) -> AsyncIterable["_models.CreatorList"]:
        """Get all Creator instances for an Azure Maps Account.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param account_name: The name of the Maps Account.
        :type account_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either CreatorList or the result of cls(response)
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.maps.models.CreatorList]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.CreatorList"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2021-02-01"
        accept = "application/json"

        def prepare_request(next_link=None):
            # Construct headers
            header_parameters = {}  # type: Dict[str, Any]
            header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

            if not next_link:
                # Construct URL
                url = self.list_by_account.metadata['url']  # type: ignore
                path_format_arguments = {
                    'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str', min_length=1),
                    'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str', max_length=90, min_length=1, pattern=r'^[-\w\._\(\)]+$'),
                    'accountName': self._serialize.url("account_name", account_name, 'str'),
                }
                url = self._client.format_url(url, **path_format_arguments)
                # Construct parameters
                query_parameters = {}  # type: Dict[str, Any]
                query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

                request = self._client.get(url, query_parameters, header_parameters)
            else:
                url = next_link
                query_parameters = {}  # type: Dict[str, Any]
                request = self._client.get(url, query_parameters, header_parameters)
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize('CreatorList', pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, response)
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(
            get_next, extract_data
        )
    list_by_account.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Maps/accounts/{accountName}/creators'}  # type: ignore

    async def create_or_update(
        self,
        resource_group_name: str,
        account_name: str,
        creator_name: str,
        creator_resource: "_models.Creator",
        **kwargs
    ) -> "_models.Creator":
        """Create or update a Maps Creator resource. Creator resource will manage Azure resources required
        to populate a custom set of mapping data. It requires an account to exist before it can be
        created.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param account_name: The name of the Maps Account.
        :type account_name: str
        :param creator_name: The name of the Maps Creator instance.
        :type creator_name: str
        :param creator_resource: The new or updated parameters for the Creator resource.
        :type creator_resource: ~azure.mgmt.maps.models.Creator
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Creator, or the result of cls(response)
        :rtype: ~azure.mgmt.maps.models.Creator
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.Creator"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2021-02-01"
        content_type = kwargs.pop("content_type", "application/json")
        accept = "application/json"

        # Construct URL
        url = self.create_or_update.metadata['url']  # type: ignore
        path_format_arguments = {
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str', min_length=1),
            'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str', max_length=90, min_length=1, pattern=r'^[-\w\._\(\)]+$'),
            'accountName': self._serialize.url("account_name", account_name, 'str'),
            'creatorName': self._serialize.url("creator_name", creator_name, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Content-Type'] = self._serialize.header("content_type", content_type, 'str')
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        body_content_kwargs = {}  # type: Dict[str, Any]
        body_content = self._serialize.body(creator_resource, 'Creator')
        body_content_kwargs['content'] = body_content
        request = self._client.put(url, query_parameters, header_parameters, **body_content_kwargs)
        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize('Creator', pipeline_response)

        if response.status_code == 201:
            deserialized = self._deserialize('Creator', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    create_or_update.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Maps/accounts/{accountName}/creators/{creatorName}'}  # type: ignore

    async def update(
        self,
        resource_group_name: str,
        account_name: str,
        creator_name: str,
        creator_update_parameters: "_models.CreatorUpdateParameters",
        **kwargs
    ) -> "_models.Creator":
        """Updates the Maps Creator resource. Only a subset of the parameters may be updated after
        creation, such as Tags.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param account_name: The name of the Maps Account.
        :type account_name: str
        :param creator_name: The name of the Maps Creator instance.
        :type creator_name: str
        :param creator_update_parameters: The update parameters for Maps Creator.
        :type creator_update_parameters: ~azure.mgmt.maps.models.CreatorUpdateParameters
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Creator, or the result of cls(response)
        :rtype: ~azure.mgmt.maps.models.Creator
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.Creator"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2021-02-01"
        content_type = kwargs.pop("content_type", "application/json")
        accept = "application/json"

        # Construct URL
        url = self.update.metadata['url']  # type: ignore
        path_format_arguments = {
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str', min_length=1),
            'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str', max_length=90, min_length=1, pattern=r'^[-\w\._\(\)]+$'),
            'accountName': self._serialize.url("account_name", account_name, 'str'),
            'creatorName': self._serialize.url("creator_name", creator_name, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Content-Type'] = self._serialize.header("content_type", content_type, 'str')
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        body_content_kwargs = {}  # type: Dict[str, Any]
        body_content = self._serialize.body(creator_update_parameters, 'CreatorUpdateParameters')
        body_content_kwargs['content'] = body_content
        request = self._client.patch(url, query_parameters, header_parameters, **body_content_kwargs)
        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('Creator', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    update.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Maps/accounts/{accountName}/creators/{creatorName}'}  # type: ignore

    async def delete(
        self,
        resource_group_name: str,
        account_name: str,
        creator_name: str,
        **kwargs
    ) -> None:
        """Delete a Maps Creator resource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param account_name: The name of the Maps Account.
        :type account_name: str
        :param creator_name: The name of the Maps Creator instance.
        :type creator_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2021-02-01"
        accept = "application/json"

        # Construct URL
        url = self.delete.metadata['url']  # type: ignore
        path_format_arguments = {
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str', min_length=1),
            'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str', max_length=90, min_length=1, pattern=r'^[-\w\._\(\)]+$'),
            'accountName': self._serialize.url("account_name", account_name, 'str'),
            'creatorName': self._serialize.url("creator_name", creator_name, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        request = self._client.delete(url, query_parameters, header_parameters)
        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    delete.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Maps/accounts/{accountName}/creators/{creatorName}'}  # type: ignore

    async def get(
        self,
        resource_group_name: str,
        account_name: str,
        creator_name: str,
        **kwargs
    ) -> "_models.Creator":
        """Get a Maps Creator resource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param account_name: The name of the Maps Account.
        :type account_name: str
        :param creator_name: The name of the Maps Creator instance.
        :type creator_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Creator, or the result of cls(response)
        :rtype: ~azure.mgmt.maps.models.Creator
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.Creator"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2021-02-01"
        accept = "application/json"

        # Construct URL
        url = self.get.metadata['url']  # type: ignore
        path_format_arguments = {
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str', min_length=1),
            'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str', max_length=90, min_length=1, pattern=r'^[-\w\._\(\)]+$'),
            'accountName': self._serialize.url("account_name", account_name, 'str'),
            'creatorName': self._serialize.url("creator_name", creator_name, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        request = self._client.get(url, query_parameters, header_parameters)
        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('Creator', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    get.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Maps/accounts/{accountName}/creators/{creatorName}'}  # type: ignore
